@@ -1,23 +1,23 @@
 from zope.component import getMultiAdapter
 from zope.interface import implements
+from zope.i18n import translate
 
 from AccessControl import Unauthorized
 from Acquisition import aq_parent, aq_inner
 from OFS.interfaces import IOrderedContainer
 from Products.ATContentTypes.interface import IATTopic
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import safe_unicode
 from Products.Five import BrowserView
 
 from plone.memoize import instance
 from plone.app.content.browser.interfaces import IFolderContentsView
-from plone.app.content.browser.tableview import Table
-from kss.core import KSSView
+from plone.app.content.browser.tableview import Table, TableKSSView
 
 from Products.CMFPlone.interfaces import IPloneSiteRoot
 
 import urllib
 
-NOT_ADDABLE_TYPES = ('Favorite',)
 
 class FolderContentsView(BrowserView):
     """
@@ -25,39 +25,39 @@ class FolderContentsView(BrowserView):
     implements(IFolderContentsView)
     
     def contents_table(self):
-        table = FolderContentsTable(self.context, self.request)
+        table = FolderContentsTable(aq_inner(self.context), self.request)
         return table.render()
 
     def title(self):
         """
         """
-        return self.context.pretty_title_or_id()
+        return aq_inner(self.context).pretty_title_or_id()
 
     def icon(self):
         """
         """
-        ploneview = getMultiAdapter((self.context, self.request), name="plone")
-        icon = ploneview.getIcon(self.context)
+        context = aq_inner(self.context)
+        ploneview = getMultiAdapter((context, self.request), name="plone")
+        icon = ploneview.getIcon(context)
         return icon.html_tag()
 
     def parent_url(self):
         """
         """
-        portal_url = getToolByName(self.context, 'portal_url')
-        plone_utils = getToolByName(self.context, 'plone_utils')
-        portal_membership = getToolByName(self.context, 'portal_membership')
+        context = aq_inner(self.context)
+        portal_membership = getToolByName(context, 'portal_membership')
 
-        obj = self.context
+        obj = context
 
         checkPermission = portal_membership.checkPermission
 
         # Abort if we are at the root of the portal
-        if IPloneSiteRoot.providedBy(self.context):
+        if IPloneSiteRoot.providedBy(context):
             return None
         
 
         # Get the parent. If we can't get it (unauthorized), use the portal
-        parent = aq_parent(aq_inner(obj))
+        parent = aq_parent(obj)
         
         # # We may get an unauthorized exception if we're not allowed to access#
         # the parent. In this case, return None
@@ -66,7 +66,7 @@ class FolderContentsView(BrowserView):
                    parent.getId() == 'talkback':
                 # Skip any Z3 views that may be in the acq tree;
                 # Skip past the talkback container if that's where we are
-                parent = aq_parent(aq_inner(parent))
+                parent = aq_parent(parent)
 
             if not checkPermission('List folder contents', parent):
                 return None
@@ -76,19 +76,6 @@ class FolderContentsView(BrowserView):
         except Unauthorized:
             return None        
 
-class FolderContentsKSSView(KSSView):
-    def update_table(self, pagenumber='1', sort_on='getObjPositionInParent', show_all=False):
-        self.request.set('sort_on', sort_on)
-        self.request.set('pagenumber', pagenumber)
-        table = FolderContentsTable(self.context, self.request,
-                                    contentFilter={'sort_on':sort_on})
-        return self.replace_table(table)
-
-    def replace_table(self, table):
-        core = self.getCommandSet('core')
-        core.replaceInnerHTML('#folderlisting-main-table', table.render())
-        return self.render()
-
 class FolderContentsTable(object):
     """   
     The foldercontents table renders the table and its actions.
@@ -99,7 +86,7 @@ class FolderContentsTable(object):
         self.request = request
         self.contentFilter = contentFilter
 
-        url = self.context.absolute_url()
+        url = context.absolute_url()
         view_url = url + '/@@folder_contents'
         self.table = Table(request, url, view_url, self.items,
                            show_sort_column=self.show_sort_column,
@@ -113,20 +100,22 @@ class FolderContentsTable(object):
     def items(self):
         """
         """
-        plone_utils = getToolByName(self.context, 'plone_utils')
-        plone_view = getMultiAdapter((self.context, self.request), name=u'plone')
-        portal_workflow = getToolByName(self.context, 'portal_workflow')
-        portal_properties = getToolByName(self.context, 'portal_properties')
+        context = aq_inner(self.context)
+        plone_utils = getToolByName(context, 'plone_utils')
+        plone_view = getMultiAdapter((context, self.request), name=u'plone')
+        portal_workflow = getToolByName(context, 'portal_workflow')
+        portal_properties = getToolByName(context, 'portal_properties')
+        portal_types = getToolByName(context, 'portal_types')
         site_properties = portal_properties.site_properties
         
         use_view_action = site_properties.getProperty('typesUseViewActionInListings', ())
-        browser_default = self.context.browserDefault()
-                
-        if IATTopic.providedBy(self.context):
-            contentsMethod = self.context.queryCatalog
+        browser_default = context.browserDefault()
+
+        if IATTopic.providedBy(context):
+            contentsMethod = context.queryCatalog
         else:
-            contentsMethod = self.context.getFolderContents
-        
+            contentsMethod = context.getFolderContents
+
         results = []
         for i, obj in enumerate(contentsMethod(self.contentFilter)):
             if (i + 1) % 2 == 0:
@@ -144,11 +133,20 @@ class FolderContentsTable(object):
             review_state = obj.review_state
             state_class = 'state-' + plone_utils.normalizeString(review_state)
             relative_url = obj.getURL(relative=True)
-            obj_type = obj.portal_type
+
+            fti = portal_types.get(obj.portal_type)
+            if fti is not None:
+                type_title_msgid = fti.Title()
+            else:
+                type_title_msgid = obj.portal_type
+            url_href_title = u'%s: %s' % (translate(type_title_msgid,
+                                                    context=self.request),
+                                          safe_unicode(obj.Description))
 
             modified = plone_view.toLocalizedTime(
                 obj.ModificationDate, long_format=1)
-            
+
+            obj_type = obj.Type
             if obj_type in use_view_action:
                 view_url = url + '/view'
             elif obj.is_folderish:
@@ -161,11 +159,11 @@ class FolderContentsTable(object):
                                  
             results.append(dict(
                 url = url,
+                url_href_title = url_href_title,
                 id  = obj.getId,
                 quoted_id = urllib.quote_plus(obj.getId),
                 path = path,
-                title_or_id = obj.pretty_title_or_id(),
-                description = obj.Description,
+                title_or_id = safe_unicode(obj.pretty_title_or_id()),
                 obj_type = obj_type,
                 size = obj.getObjSize,
                 modified = modified,
@@ -180,15 +178,15 @@ class FolderContentsTable(object):
                 relative_url = relative_url,
                 view_url = view_url,
                 table_row_class = table_row_class,
-                is_expired = self.context.isExpired(obj),
+                is_expired = context.isExpired(obj),
             ))
         return results
 
     @property
     def orderable(self):
         """
-        """        
-        return IOrderedContainer.providedBy(self.context)
+        """
+        return IOrderedContainer.providedBy(aq_inner(self.context))
 
     @property
     def show_sort_column(self):
@@ -198,15 +196,16 @@ class FolderContentsTable(object):
     def editable(self):
         """
         """
-        context_state = getMultiAdapter((self.context, self.request),
+        context_state = getMultiAdapter((aq_inner(self.context), self.request),
                                         name=u'plone_context_state')
         return context_state.is_editable()
 
     @property
     def buttons(self):
         buttons = []
-        portal_actions = getToolByName(self.context, 'portal_actions')
-        button_actions = portal_actions.listActionInfos(object=aq_inner(self.context), categories=('folder_buttons', ))
+        context = aq_inner(self.context)
+        portal_actions = getToolByName(context, 'portal_actions')
+        button_actions = portal_actions.listActionInfos(object=context, categories=('folder_buttons', ))
 
         # Do not show buttons if there is no data, unless there is data to be
         # pasted
@@ -220,7 +219,7 @@ class FolderContentsTable(object):
 
         for button in button_actions:
             # Make proper classes for our buttons
-            if button['id'] != 'paste' or self.context.cb_dataValid():
+            if button['id'] != 'paste' or context.cb_dataValid():
                 buttons.append(self.setbuttonclass(button))
         return buttons
 
@@ -230,3 +229,7 @@ class FolderContentsTable(object):
         else:
             button['cssclass'] = 'context'
         return button
+
+class FolderContentsKSSView(TableKSSView):
+    table = FolderContentsTable
+
