@@ -1,6 +1,17 @@
 from zope.app.pagetemplate import ViewPageTemplateFile
 from plone.app.content.batching import Batch
 from plone.memoize import instance
+from zope.i18nmessageid import MessageFactory
+
+from Products.CMFPlone.utils import safe_unicode
+
+_ = MessageFactory('plone')
+
+try:
+    from kss.core import KSSView
+except ImportError:
+    from Products.Five import BrowserView as KSSView
+
 
 class Table(object):
     """   
@@ -32,6 +43,14 @@ class Table(object):
 
         self.pagenumber =  int(request.get('pagenumber', 1))
 
+    def msg_select_item(self, item):
+        return _(u'checkbox_select_item',
+                 default=u"Select ${title}",
+                 mapping={'title': safe_unicode(item['title_or_id'])})
+
+    @property
+    def within_batch_size(self):
+        return len(self.items) < self.pagesize
 
     def set_checked(self, item):
         selected = self.selected(item)
@@ -117,3 +136,20 @@ class Table(object):
     @property
     def viewname(self):
         return self.view_url.split('?')[0].split('/')[-1]
+
+class TableKSSView(KSSView):
+    '''Base class which can be used from a KSS view
+
+    Subclasses only need to set the table property to a different
+    class.'''
+    
+    table = None
+
+    def update_table(self, pagenumber='1', sort_on='getObjPositionInParent', show_all=False):
+        self.request.set('sort_on', sort_on)
+        self.request.set('pagenumber', pagenumber)
+        table = self.table(self.context, self.request,
+                                    contentFilter={'sort_on':sort_on})
+        core = self.getCommandSet('core')
+        core.replaceHTML('#folderlisting-main-table', table.render())
+        return self.render()
